@@ -15,6 +15,10 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 #include "mountlist.h"
@@ -36,6 +40,10 @@ char *xrealloc ();
 char *xstrdup ();
 void error ();
 
+#ifdef HAVE_SYS_PARAM_H
+#include <sys/param.h>
+#endif
+
 #if defined (MOUNTED_GETFSSTAT)	/* __alpha running OSF_1 */
 #  include <sys/mount.h>
 #  include <sys/fs_types.h>
@@ -58,7 +66,6 @@ void error ();
 #endif
 
 #ifdef MOUNTED_GETMNT		/* Ultrix.  */
-#include <sys/param.h>
 #include <sys/mount.h>
 #include <sys/fs_types.h>
 #endif
@@ -98,7 +105,7 @@ xatoi (cp)
      char *cp;
 {
   int val;
-  
+
   val = 0;
   while (*cp)
     {
@@ -116,7 +123,7 @@ xatoi (cp)
 }
 #endif /* MOUNTED_GETMNTENT1.  */
 
-#ifdef MOUNTED_GETMNTINFO	/* 4.4BSD.  */
+#if defined (MOUNTED_GETMNTINFO) && !defined (__NetBSD__)
 static char *
 fstype_to_string (t)
      short t;
@@ -244,7 +251,11 @@ read_filesystem_list (need_fs_type, all_fs)
 	me = (struct mount_entry *) xmalloc (sizeof (struct mount_entry));
 	me->me_devname = xstrdup (fsp->f_mntfromname);
 	me->me_mountdir = xstrdup (fsp->f_mntonname);
+#ifdef __NetBSD__
+	me->me_type = xstrdup (fsp->f_fstypename);
+#else
 	me->me_type = fstype_to_string (fsp->f_type);
+#endif
 	me->me_dev = -1;	/* Magic; means not known yet. */
 	me->me_next = NULL;
 
@@ -375,7 +386,7 @@ read_filesystem_list (need_fs_type, all_fs)
 	me->me_type =  xstrdup ((*ent)->mt_fstype);
 	me->me_dev = -1;	/* Magic; means not known yet. */
 	me->me_next = NULL;
-	
+
 	/* Add to the linked list. */
 	mtail->me_next = me;
 	mtail = me;
@@ -448,7 +459,7 @@ read_filesystem_list (need_fs_type, all_fs)
 	  }
 	else
 	  {
-	    me->me_devname = xstrdup (thisent + 
+	    me->me_devname = xstrdup (thisent +
 				      vmp->vmt_data[VMT_OBJECT].vmt_off);
 	  }
 	me->me_mountdir = xstrdup (thisent + vmp->vmt_data[VMT_STUB].vmt_off);
