@@ -28,17 +28,7 @@
 
    Written by David MacKenzie <djm@gnu.ai.mit.edu>. */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
 #include <ctype.h>
 #include <sys/types.h>
@@ -47,6 +37,7 @@
 #include <getopt.h>
 #include "system.h"
 #include "version.h"
+#include "safe-lstat.h"
 
 #ifndef _POSIX_VERSION
 struct passwd *getpwnam ();
@@ -59,9 +50,9 @@ struct group *getgrgid ();
 #define endpwent()
 #endif
 
-int lstat ();
 char *savedir ();
 char *parse_user_spec ();
+void strip_trailing_slashes ();
 char *xmalloc ();
 char *xrealloc ();
 void error ();
@@ -160,7 +151,10 @@ main (argc, argv)
     usage (0);
 
   if (optind >= argc - 1)
-    usage (1);
+    {
+      error (0, 0, "too few arguments");
+      usage (1);
+    }
 
   e = parse_user_spec (argv[optind], &user, &group, &username, &groupname);
   if (e)
@@ -169,7 +163,10 @@ main (argc, argv)
     username = "";
 
   for (++optind; optind < argc; ++optind)
-    errors |= change_file_owner (argv[optind], user, group);
+    {
+      strip_trailing_slashes (argv[optind]);
+      errors |= change_file_owner (argv[optind], user, group);
+    }
 
   exit (errors);
 }
@@ -189,7 +186,7 @@ change_file_owner (file, user, group)
   gid_t newgroup;
   int errors = 0;
 
-  if (lstat (file, &file_stats))
+  if (SAFE_LSTAT (file, &file_stats))
     {
       if (force_silent == 0)
 	error (0, errno, "%s", file);
