@@ -40,25 +40,14 @@
  #pragma alloca
 #endif
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
 #include "version.h"
-
-int lstat ();
-int stat ();
+#include "safe-stat.h"
+#include "safe-lstat.h"
 
 /* Initial number of entries in each hash table entry's table of inodes.  */
 #define INITIAL_HASH_MODULE 100
@@ -101,7 +90,9 @@ typedef struct
 } *string, stringstruct;
 
 char *savedir ();
+#ifndef HAVE_FCHDIR
 char *xgetcwd ();
+#endif
 char *xmalloc ();
 char *xrealloc ();
 void error ();
@@ -173,6 +164,9 @@ static int show_help;
 /* If non-zero, print the version on standard output and exit.  */
 static int show_version;
 
+/* Grand total size of all args. */
+static long tot_size = 0L;
+
 static struct option const long_options[] =
 {
   {"all", no_argument, &opt_all, 1},
@@ -229,9 +223,13 @@ main (argc, argv)
      char *argv[];
 {
   int c;
+  char *cwd_only[2];
+  
+  cwd_only[0] = ".";
+  cwd_only[1] = NULL;
 
   program_name = argv[0];
-  xstat = lstat;
+  xstat = safe_lstat;
   output_size = getenv ("POSIXLY_CORRECT") ? size_blocks : size_kilobytes;
 
   while ((c = getopt_long (argc, argv, "abcklsxDLS", long_options, (int *) 0))
@@ -275,7 +273,7 @@ main (argc, argv)
 	  break;
 
 	case 'L':
-	  xstat = stat;
+	  xstat = safe_stat;
 	  break;
 
 	case 'S':
@@ -304,20 +302,7 @@ main (argc, argv)
 
   str_init (&path, INITIAL_PATH_SIZE);
 
-  if (optind == argc)
-    {
-      str_copyc (path, ".");
-
-      /* Initialize the hash structure for inode numbers.  */
-      hash_reset ();
-
-      /* Get the size of the current directory only.  */
-      count_entry (".", 1, 0);
-    }
-  else
-    {
-      du_files (argv + optind);
-    }
+  du_files (optind == argc ? cwd_only : argv + optind);
 
   exit (exit_status);
 }
@@ -329,18 +314,27 @@ static void
 du_files (files)
      char **files;
 {
+#ifdef HAVE_FCHDIR
+  int wd_desc;
+#else
   char *wd;
+#endif
   ino_t initial_ino;		/* Initial directory's inode. */
   dev_t initial_dev;		/* Initial directory's device. */
-  long tot_size = 0L;		/* Grand total size of all args. */
   int i;			/* Index in FILES. */
 
+#ifdef HAVE_FCHDIR
+  wd_desc = open (".", O_RDONLY);
+  if (wd_desc < 0)
+    error (1, errno, "cannot open current directory");
+#else
   wd = xgetcwd ();
   if (wd == NULL)
     error (1, errno, "cannot get current directory");
+#endif
 
   /* Remember the inode and device number of the current directory.  */
-  if (stat (".", &stat_buf))
+  if (SAFE_STAT (".", &stat_buf))
     error (1, errno, "current directory");
   initial_ino = stat_buf.st_ino;
   initial_dev = stat_buf.st_dev;
@@ -369,14 +363,19 @@ du_files (files)
       if (!opt_combined_arguments)
 	hash_reset ();
 
-      tot_size += count_entry (arg, 1, 0);
+      count_entry (arg, 1, 0);
 
       /* chdir if `count_entry' has changed the working directory.  */
-      if (stat (".", &stat_buf))
+      if (SAFE_STAT (".", &stat_buf))
 	error (1, errno, ".");
       if ((stat_buf.st_ino != initial_ino || stat_buf.st_dev != initial_dev)
+#ifdef HAVE_FCHDIR
+	  && fchdir (wd_desc) < 0)
+	error (1, errno, "cannot return to original directory");
+#else
 	  && chdir (wd) < 0)
 	error (1, errno, "cannot change to directory %s", wd);
+#endif
     }
 
   if (opt_combined_arguments)
@@ -386,7 +385,9 @@ du_files (files)
       fflush (stdout);
     }
 
+#ifndef HAVE_FCHDIR
   free (wd);
+#endif
 }
 
 /* Print (if appropriate) and return the size
@@ -402,9 +403,9 @@ count_entry (ent, top, last_dev)
 {
   long size;
 
-  if ((top && opt_dereference_arguments ?
-      stat (ent, &stat_buf) :
-      (*xstat) (ent, &stat_buf)) < 0)
+  if (((top && opt_dereference_arguments)
+       ? SAFE_STAT (ent, &stat_buf)
+       : (*xstat) (ent, &stat_buf)) < 0)
     {
       error (0, errno, "%s", path->text);
       exit_status = 1;
@@ -421,6 +422,8 @@ count_entry (ent, top, last_dev)
   else
     size = ST_NBLOCKS (stat_buf);
 
+  tot_size += size;
+
   if (S_ISDIR (stat_buf.st_mode))
     {
       unsigned pathlen;
@@ -481,7 +484,7 @@ count_entry (ent, top, last_dev)
 	{
 	  printf ("%ld\t%s\n", output_size == size_bytes ? size
 		  : convert_blocks (size, output_size == size_kilobytes),
-		  path->text);
+		  path->length > 0 ? path->text : "/");
 	  fflush (stdout);
 	}
       return opt_separate_dirs ? 0 : size;
