@@ -52,17 +52,7 @@
 
    David MacKenzie <djm@gnu.ai.mit.edu> */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
 #include <getopt.h>
 #include <ctype.h>
@@ -71,7 +61,9 @@
 #include <grp.h>
 #include "system.h"
 #include "version.h"
+#include "safe-stat.h"
 #include "modechange.h"
+#include "makepath.h"
 
 #if !defined (isascii) || defined (STDC_HEADERS)
 #undef isascii
@@ -105,7 +97,8 @@ char *basename ();
 char *stpcpy ();
 char *xmalloc ();
 void error ();
-int make_path ();
+int safe_read ();
+int full_write ();
 int isdir ();
 
 static int change_attributes ();
@@ -218,10 +211,15 @@ main (argc, argv)
     usage (0);
 
   /* Check for invalid combinations of arguments. */
-  if ((dir_arg && strip_files)
-      || (optind == argc)
-      || (optind == argc - 1 && !dir_arg))
-    usage (1);
+  if (dir_arg && strip_files)
+    error (1, 0,
+	   "the strip option may not be used when installing a directory");
+
+  if (optind == argc || (optind == argc - 1 && !dir_arg))
+    {
+      error (0, 0, "too few arguments");
+      usage (1);
+    }
 
   if (symbolic_mode)
     {
@@ -240,7 +238,7 @@ main (argc, argv)
       for (; optind < argc; ++optind)
 	{
 	  errors |=
-	    make_path (argv[optind], mode, mode, owner_id, group_id, NULL);
+	    make_path (argv[optind], mode, mode, owner_id, group_id, 0, NULL);
 	}
     }
   else
@@ -330,7 +328,7 @@ copy_file (from, to, to_created)
   struct stat from_stats, to_stats;
   int target_created = 1;
 
-  if (stat (from, &from_stats))
+  if (SAFE_STAT (from, &from_stats))
     {
       error (0, errno, "%s", from);
       return 1;
@@ -340,7 +338,7 @@ copy_file (from, to, to_created)
       error (0, 0, "`%s' is not a regular file", from);
       return 1;
     }
-  if (stat (to, &to_stats) == 0)
+  if (SAFE_STAT (to, &to_stats) == 0)
     {
       if (!S_ISREG (to_stats.st_mode))
 	{
@@ -374,8 +372,8 @@ copy_file (from, to, to_created)
       return 1;
     }
 
-  while ((bytes = read (fromfd, buffer, READ_SIZE)) > 0)
-    if (write (tofd, buffer, bytes) != bytes)
+  while ((bytes = safe_read (fromfd, buffer, READ_SIZE)) > 0)
+    if (full_write (tofd, buffer, bytes) < 0)
       {
 	error (0, errno, "%s", to);
 	goto copy_error;
@@ -555,9 +553,9 @@ Usage: %s [OPTION]... SOURCE DEST           (1st format)\n\
 \n\
   -c                  (ignored)\n\
   -d, --directory     create [leading] directories, mandatory for 3rd format\n\
-  -g, --group GROUP   set group ownership, instead of process' current group\n\
-  -m, --mode MODE     set permission mode (as in chmod), instead of 0755\n\
-  -o, --owner OWNER   set ownership (super-user only)\n\
+  -g, --group=GROUP   set group ownership, instead of process' current group\n\
+  -m, --mode=MODE     set permission mode (as in chmod), instead of 0755\n\
+  -o, --owner=OWNER   set ownership (super-user only)\n\
   -s, --strip         strip symbol tables, only for 1st and 2nd formats\n\
       --help          display this help and exit\n\
       --version       output version information and exit\n");
