@@ -22,17 +22,7 @@
 
    David MacKenzie <djm@ai.mit.edu>  */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
@@ -102,7 +92,10 @@ main (argc, argv)
     usage (0);
 
   if (optind == argc)
-    usage (1);
+    {
+      error (0, 0, "too few arguments");
+      usage (1);
+    }
 
   newmode = 0666 & ~umask (0);
   if (symbolic_mode)
@@ -141,7 +134,7 @@ usage (status)
       printf ("Usage: %s [OPTION] PATH...\n", program_name);
       printf ("\
 \n\
-  -m, --mode MODE   set permission mode (as in chmod), not 0666 - umask\n\
+  -m, --mode=MODE   set permission mode (as in chmod), not 0666 - umask\n\
       --help        display this help and exit\n\
       --version     output version information and exit\n");
     }
