@@ -41,23 +41,14 @@
  #pragma alloca
 #endif
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
 #include "backupfile.h"
 #include "version.h"
+#include "safe-lstat.h"
 
 #ifndef _POSIX_VERSION
 uid_t geteuid ();
@@ -68,6 +59,8 @@ enum backup_type get_version ();
 int isdir ();
 int yesno ();
 void error ();
+int safe_read ();
+int full_write ();
 void strip_trailing_slashes ();
 int eaccess_stat ();
 char *stpcpy ();
@@ -184,7 +177,10 @@ main (argc, argv)
     usage (0);
 
   if (argc < optind + 2)
-    usage (1);
+    {
+      error (0, 0, "missing file argument%s", argc == optind ? "s" : "");
+      usage (1);
+    }
 
   if (make_backups)
     backup_type = get_version (version);
@@ -209,7 +205,7 @@ is_real_dir (path)
 {
   struct stat stats;
 
-  return lstat (path, &stats) == 0 && S_ISDIR (stats.st_mode);
+  return SAFE_LSTAT (path, &stats) == 0 && S_ISDIR (stats.st_mode);
 }
 
 /* Move file SOURCE onto DEST.  Handles the case when DEST is a directory.
@@ -251,13 +247,13 @@ do_move (source, dest)
 {
   char *dest_backup = NULL;
 
-  if (lstat (source, &source_stats) != 0)
+  if (SAFE_LSTAT (source, &source_stats) != 0)
     {
       error (0, errno, "%s", source);
       return 1;
     }
 
-  if (lstat (dest, &dest_stats) == 0)
+  if (SAFE_LSTAT (dest, &dest_stats) == 0)
     {
       if (source_stats.st_dev == dest_stats.st_dev
 	  && source_stats.st_ino == dest_stats.st_ino)
@@ -277,7 +273,7 @@ do_move (source, dest)
 	return 0;
 
       if (!override_mode && (interactive || stdin_tty)
-	  && eaccess_stat (&dest_stats, W_OK))
+	  && eaccess_stat (&dest_stats, W_OK, dest))
 	{
 	  fprintf (stderr, "%s: replace `%s', overriding mode %04o? ",
 		   program_name, dest,
@@ -393,25 +389,16 @@ copy_reg (source, dest)
       return 1;
     }
 
-  while ((len = read (ifd, buf, sizeof (buf))) > 0)
+  while ((len = safe_read (ifd, buf, sizeof (buf))) > 0)
     {
-      int wrote = 0;
-      char *bp = buf;
-
-      do
+      if (full_write (ofd, buf, len) < 0)
 	{
-	  wrote = write (ofd, bp, len);
-	  if (wrote < 0)
-	    {
-	      error (0, errno, "%s", dest);
-	      close (ifd);
-	      close (ofd);
-	      unlink (dest);
-	      return 1;
-	    }
-	  bp += wrote;
-	  len -= wrote;
-	} while (len > 0);
+	  error (0, errno, "%s", dest);
+	  close (ifd);
+	  close (ofd);
+	  unlink (dest);
+	  return 1;
+	}
     }
   if (len < 0)
     {
@@ -489,8 +476,8 @@ Usage: %s [OPTION]... SOURCE DEST\n\
   -i, --interactive            prompt before overwrite\n\
   -u, --update                 move only older or brand new files\n\
   -v, --verbose                explain what is being done\n\
-  -S, --suffix SUFFIX          override the usual backup suffix\n\
-  -V, --version-control WORD   override the usual version control\n\
+  -S, --suffix=SUFFIX          override the usual backup suffix\n\
+  -V, --version-control=WORD   override the usual version control\n\
       --help                   display this help and exit\n\
       --version                output version information and exit\n\
 \n\
