@@ -28,12 +28,14 @@
    Ian Dall (ian@sibyl.eleceng.ua.oz.au)
    and David MacKenzie (djm@gnu.ai.mit.edu) */
 
+#include <config.h>
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include <signal.h>
 #include "system.h"
 #include "version.h"
+#include "safe-stat.h"
 
 #ifndef HIPRI
 #define HIPRI -10
@@ -58,24 +60,34 @@ void strip_trailing_slashes ();
 static char *fullpath ();
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output and exit.  */
+static int show_version;
 
 static struct option const long_options[] =
 {
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {0, 0, 0, 0}
 };
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "Usage: %s [--help] [--version] existing-dir new-dir\n",
-	   program_name);
-  exit (2);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("Usage: %s [OPTION]... EXISTING_DIR NEW_DIR\n", program_name);
+      printf ("\
+\n\
+   --help      display this help and exit\n\
+   --version   output version information and exit\n");
+    }
+  exit (status);
 }
 
 void
@@ -99,21 +111,21 @@ main (argc, argv)
 	  break;
 
 	default:
-	  usage ();
+	  usage (2);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("%s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (argc - optind != 2)
-    usage ();
+    usage (2);
 
   from = argv[optind];
   to = argv[optind + 1];
@@ -128,13 +140,13 @@ main (argc, argv)
   from_base = basename (from);
   if (!strcmp (from_base, ".") || !strcmp (from_base, ".."))
     error (1, 0, "cannot rename `.' or `..'");
-  
+
   /* Even with an effective uid of root, link fails if the target exists.
      That is what we want, so don't unlink `to' first.
      However, we do need to check that the directories that link and unlink
      will modify exist and are writable by the user. */
 
-  if (stat (from, &from_stats))
+  if (SAFE_STAT (from, &from_stats))
     error (1, errno, "%s", from);
   if (!S_ISDIR (from_stats.st_mode))
     error (1, 0, "`%s' is not a directory", from);
@@ -146,7 +158,7 @@ main (argc, argv)
   /* To prevent disconnecting the tree rooted at `from' from its parent,
      quit if any of the directories in `to' are the same (dev and ino)
      as the directory `from'. */
-  
+
   slash = to_parent_path = fullpath (to_parent);
   while (1)
     {
@@ -156,7 +168,7 @@ main (argc, argv)
       if (slash != NULL)
 	*slash = '\0';
 
-      if (stat (to_parent_path, &to_stats))
+      if (SAFE_STAT (to_parent_path, &to_stats))
 	error (1, errno, "%s", to_parent_path);
       if (to_stats.st_dev == from_stats.st_dev
 	  && to_stats.st_ino == from_stats.st_ino)
