@@ -1,5 +1,5 @@
 /* `rm' file deletion utility for GNU.
-   Copyright (C) 1988, 1990, 1991 Free Software Foundation, Inc.
+   Copyright (C) 1988, 1990, 1991, 1994 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,29 +17,19 @@
 
 /* Written by Paul Rubin, David MacKenzie, and Richard Stallman. */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
-
 #include <stdio.h>
 #include <getopt.h>
 #include <sys/types.h>
 #include "system.h"
 #include "version.h"
+#include "safe-lstat.h"
 
-#ifdef _POSIX_SOURCE
-/* POSIX.1 doesn't have inodes, so fake them to avoid lots of ifdefs. */
-#define ino_t unsigned long
-#define D_INO(dp) 1
-#else
+#ifdef D_INO_IN_DIRENT
 #define D_INO(dp) ((dp)->d_ino)
+#else
+/* POSIX.1 doesn't have inodes, so fake them to avoid lots of ifdefs. */
+#define D_INO(dp) 1
 #endif
 
 char *basename ();
@@ -163,7 +153,10 @@ main (argc, argv)
       if (ignore_missing_files)
 	exit (0);
       else
-	usage (1);
+	{
+	  error (0, 0, "too few arguments");
+	  usage (1);
+	}
     }
 
   stdin_tty = isatty (0);
@@ -205,7 +198,7 @@ rm ()
       return 1;
     }
 
-  if (lstat (pathname, &path_stats))
+  if (SAFE_LSTAT (pathname, &path_stats))
     {
       if (errno == ENOENT && ignore_missing_files)
 	return 0;
@@ -228,7 +221,7 @@ remove_file (statp)
      struct stat *statp;
 {
   if (!ignore_missing_files && (interactive || stdin_tty)
-      && eaccess_stat (statp, W_OK)
+      && eaccess_stat (statp, W_OK, pathname)
 #ifdef S_ISLNK
       && !S_ISLNK (statp->st_mode)
 #endif
@@ -280,7 +273,7 @@ remove_dir (statp)
     }
 
   if (!ignore_missing_files && (interactive || stdin_tty)
-      && eaccess_stat (statp, W_OK))
+      && eaccess_stat (statp, W_OK, pathname))
     {
       fprintf (stderr,
 	       "%s: descend directory `%s', overriding mode %04o? ",
